#include <stdio.h>
#include <shmem.h>
#include "pgmio.h"

#define nx 360
#define ny 240
#define numpes 8
#define nxlocal (nx/numpes)
#define nylocal ny

double bigpic[nx][ny];
double smallpic[nxlocal][nylocal];

int main(void)
{
  
  int ix, iy;
  int pe, mype;

  shmem_init();

  mype = shmem_my_pe();

  if (numpes != shmem_n_pes())
  {

    if (mype == 0)
    {
      printf("Error: compiled for %d PE(s), but running on %d\n", numpes, shmem_n_pes());
    }

    shmem_finalize();

    return -1;

  }

  if (mype == 0)
  {
    printf("Running with %d PE(s)\n",numpes);
  }

  printf("Hello from PE %d out of %d\n",mype,numpes);

  for (ix=0; ix < nx; ix++)
  {
    for (iy=0; iy < ny; iy++)
    {
      bigpic[ix][iy] = -1;
    }
  }

  for (ix=0; ix < nxlocal; ix++)
  {
    for (iy=0; iy < nylocal; iy++)
    {
      smallpic[ix][iy] = mype;
    }
  }

  /* Need to synchronise to ensure smallpic is initialised on all PEs
     before the master PE reads them. */

  shmem_barrier_all();

  if (mype == 0)
  {

    for (pe=0; pe < numpes; pe++)
    {
      shmem_double_get(&bigpic[pe*nxlocal][0], &smallpic[0][0], nxlocal*nylocal, pe);
    }

    pgmwrite("output.pgm", bigpic, nx, ny);

  }

  shmem_finalize();

  return 0;
}

