#include <stdio.h>
#include <shmem.h>
#include "pgmio.h"

#define nx 360
#define ny 240
#define numpes 8
#define nxlocal (nx/numpes)
#define nylocal ny

double bigpic[nx][ny];
double oldpic[nxlocal+2][nylocal+2];
double newpic[nxlocal+2][nylocal+2];

int main(void)
{
  
  int ix, iy;
  int mype;

  shmem_init();

  mype = shmem_my_pe();

  if (numpes != shmem_n_pes())
  {

    if (mype == 0)
    {
      printf("Error: compiled for %d PE(s), but running on %d\n", numpes, shmem_n_pes());
    }

    shmem_finalize();

    return -1;

  }

  if (mype == 0)
  {
    printf("Running with %d PE(s)\n",numpes);
  }

  printf("Hello from PE %d out of %d\n",mype,numpes);

  for (ix=0; ix < (nxlocal+2); ix++)
  {
    for (iy=0; iy < (nylocal+2); iy++)
    {
      oldpic[ix][iy] = 0.0;
      newpic[ix][iy] = 0.0;
    }
  }

  /* Read the picture on the master and copy to oldpic */

  if (mype == 0)
  {

    pgmread("picture360x240.pgm", bigpic, nx, ny);

    /* The following is only correct on a single PE - you need to change this */

    for(ix = 0; ix < nxlocal; ix++)
    {
      for(iy = 0; iy < nylocal; iy++)
      {
        oldpic[ix+1][iy+1] = bigpic[ix][iy];
      }
    }

  }

  /* Overwrite bigpic to ensure that we have to do the output correctly */
  for (ix=0; ix < nx; ix++)
  {
    for (iy=0; iy < ny; iy++)
    {
      bigpic[ix][iy] = -1.0;
    }
  }

  /* This is where we should do the actual calculation of newpic as the
     second derivative (ie the edges) of oldpic. In this template, simply
     set newpic = oldpic. */

  for(ix = 0; ix < (nxlocal+2); ix++)
  {
    for(iy = 0; iy < (nylocal+2); iy++)
    {
      newpic[ix][iy] = oldpic[ix][iy];
    }
  }

  /* Now write the output on the master PE. */

  if (mype == 0)
  {

    /* Copy newpic back to bigpic */
    for(ix = 0; ix < nxlocal; ix++)
    {
      for(iy = 0; iy < nylocal; iy++)
      {
        bigpic[ix][iy] = newpic[ix+1][iy+1];
      }
    }

    pgmwrite("output.pgm", bigpic, nx, ny);

  }

  shmem_finalize();

  return 0;
}

